module Jabysoft
  class BaseController < ApplicationController
    # This class works as a Restfull CRUD Controller to provide the same crud functions to all subcontrollers
    protect_from_forgery with: :exception
    respond_to :json, :html, :xml
    
    # Callbacks here
    before_action :set_resource, only: [:show, :edit, :update, :destroy]
    before_action :disable_edition, only: [:show]
    before_action :load_form_resources, except: [:index, :destroy]
    
    PER_PAGE = 8

    def index
      if included_attributes
        instance_variable_set(collection_instance_variable, model_class.includes(included_attributes).paginate(page: params[:page], per_page: PER_PAGE))
      else
        instance_variable_set(collection_instance_variable, model_class.paginate(page: params[:page], per_page: PER_PAGE))
      end
      search
      respond_with collection_variable
    end

    def show; end
    def edit; end

    def new
      instance_variable_set(instance_variable, model_class.new)
    end

    def create
      instance_variable_set(instance_variable, model_class.new(permit_params))
      flash[:success] = I18n.t('created', model: model_class.model_name.human) if variable.save
      responds :create
    end

    def update
      flash[:success] = I18n.t('updated', model: model_class.model_name.human) if variable.update(permit_params)
      responds :update
    end

    def destroy
      flash[:success] = I18n.t('destroyed', model: model_class.model_name.human) if variable.destroy
      redirect_to index_path
    end

    protected
    # to be used by subclasses
    def included_attributes;end

    # to be used by subclasses
    def load_form_resources;end

    # to be used by subclasses
    def after_create;end

    # to be used by subclasses
    def after_update;end

    def model_class
      get_model_name.singularize.constantize
    end

    def index_path
      Rails.application.routes.url_helpers.send("#{get_model_name.underscore}_path")
    end

    def get_model_name
      model_name = self.class.name
      model_name.slice! 'Controller'
      model_name
    end

    def disable_edition
      @readonly = true
    end

    def template_not_found
      redirect_to '/404.html'
    end

    private
    def search
      return unless params[:search].present?
      result = collection_variable.search(params[:search])
      instance_variable_set(collection_instance_variable, result)
    end

    def variable
      instance_variable_get(instance_variable)
    end

    def collection_variable
      instance_variable_get(collection_instance_variable)
    end

    def controller_namespace
      params[:controller].split('/').first if params[:controller].include?('/')
    end

    def instance_variable
      "@#{model_class_name}"
    end

    def collection_instance_variable
      "@#{model_class_name.pluralize}"
    end

    def model_class_name
      model_class.name.underscore
    end

    def set_resource
      instance_variable_set(instance_variable, model_class.find(params[:id]))
    end

    def responds(action)
      message = "after_#{action}".to_sym
      return send(message, variable.valid?) if respond_to?(message)

      if controller_namespace
        respond_with controller_namespace.to_sym, variable
      else
        respond_with variable
      end
    end
  end
end