module BaseHelper
  def translate_attribute(model, attribute = nil)
    return model.human_attribute_name(attribute) if attribute
    model.model_name.human
  end

  alias ta translate_attribute

  def edit_url_for(model, namespace: nil)
    if model.id
      _namespace_ = namespace ? namespace.concat('_') : namespace
      path = "edit_#{_namespace_}#{model.model_name.to_s.underscore}_path"
      Rails.application.routes.url_helpers.send(path.to_sym, id: model.id)
    end
  end

  def base_path(model)
    "/#{model.model_name.name.underscore.pluralize}"
  end

  def remote_request(type, path, params={}, target_tag_id = nil)
    if target_tag_id
      "$.#{type}('#{path}',
               { #{collect_request_params(params)} },
               function(data) {$('##{target_tag_id}').html(data);}
      );"
    else
      "$.#{type}('#{path}',
               { #{collect_request_params(params)} }
      );"
    end
  end

  def boolean_image(boolean)
    if boolean
      "<span class='glyphicon glyphicon-ok green'>".html_safe
    else
      "<span class='glyphicon glyphicon-remove red'>".html_safe
    end
  end

  def short_date(date)
    return "" unless date
    l(date, format: :short)
  end

  def price(value)
    number_to_currency(value)
  end

  private
  def collect_request_params(params)
    params.collect { |param| "#{param[0]}: #{param[1]}" }.join(", ")
  end
end
