module LinksHelper
  include Jabysoft::AuthorisationService

  def link_to_new(model, url, html_options = {})
    authorise_link(:create, model: model, style: 'btn btn-primary btn-lg', options: html_options) do |authorised|
      if authorised
        link_to t('new', model: model.model_name.human), url, html_options
      else
        content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
          link_to t('new', model: model.model_name.human), url, html_options
        end
      end
    end
  end

  def link_to_edit(model, url, html_options = {})
    authorise_link(:update, model: model, 
      style: 'btn btn-default glyphicon glyphicon-pencil', options: html_options) do |authorised|
      if authorised
        link_to '', url, html_options
      else
        content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
          link_to t('edit'), url, html_options
        end
      end
    end
  end

  def link_to_show(model, url, html_options = {})
    style = 'btn btn-default glyphicon glyphicon-zoom-in'
    authorise_link(:show, model: model, style: style, options: html_options) do |authorised|
      if authorised
        link_to '', url, html_options
      else
        content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
          link_to t('show'), url, html_options
        end
      end
    end
  end

  def link_to_destroy(model, url, html_options = {})
    style = 'btn btn-danger glyphicon glyphicon-trash'
    resource_id = html_options[:resource_id].presence
    html_options.reverse_merge!({"data-target" => "#delete-confirmation#{resource_id}", "data-toggle" => "modal"})
    authorise_link(:destroy, model: model, style: style, options: html_options) do |authorised|
      if authorised
        link_to '', url, html_options
      else
        content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
          link_to t('destroy'), url, html_options
        end
      end
    end
  end

  def link_to_back(url, html_options = {})
    html_options.reverse_merge!(class: 'btn btn-default btn-lg')
    link_to t('back'), url, html_options
  end

  def link_to_menu(model, url)
    link_to model.model_name.human.pluralize, url
  end

  def submit_or_edit_button(form, edit_path, is_edition)
    if is_edition && params[:action] != 'new'
      link_to_edit form.object, edit_path, class: "btn btn-primary glyphicon glyphicon-pencil"
    else
      submit_button(form)
    end
  end

  def submit_button(form, value = nil)
    if value
      form.submit value, class: "btn btn-primary btn-lg"
    else
      form.submit t('save'), class: "btn btn-primary btn-lg"
    end
  end
end
